;     ******************************************************************
;     *                                                                *
;     *                            EvilFX05.asm                        *
;     *                                                                *
;     *                          EvilFx Processor                      *
;     *                                                                *
;     *        (c) 1998 by S.Pfeiffer     ver 0.3  08.01.98            *
;     *                                                                *
;     ******************************************************************
;
;
;    - for single Motorola EVM56k Board (>=66MHz)
;    - PortB usage : Fx Type Switch (0..3)    inputs
;                    Fx Parameter 1 (4..7)     "
;                    Fx Parameter 2 (8..11)    "
;                    Level LEDs     (12..13)  outputs
;    - 2 channel (stereo) input via codec
;    - stereo / 2 channel processing
;    - 2 channel output (via codec) :
;
;    ver 0.3 :  Structure works
;               Quantizer works
;               Delay Line works
;		Flash.asm routine at $3E00 saves the executable
;		in the optional Flash Rom with autostart
;******************************************************************************

; Structure
;
;- Codec controls transfer of samples via interrupt (2*4 IRQs per Cycle)
;- signal processing at end of SSI_TX_ISR
; -> SSI_RX_ISR interrupts TX_ISR 4 times per cycle
; -> SSI_TX_ISR interrupts itself 3 times

;******************************************************************************

Fs      set     48000.0   ;Specify sampling frequency.
                          ; this is the Codec samp. freq.              
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
LED_time        EQU     $2000   ; LED-lighting time in cycles
;----------------------------------------------------------------------------
;******************************************************************************

; **************************************************************************
; *                           CODEC . ASM                                  *
; *                 !!  specially adapted version !!                       *
; **************************************************************************

NO_PREAMP       equ     $100000 
LO_OUT_DRV      equ     $080000
HI_PASS_FILT    equ     $008000
SAMP_RATE_9     equ     $003800
SAMP_RATE_48    equ     $003000
SAMP_RATE_32    equ     $001800
SAMP_RATE_27    equ     $001000
SAMP_RATE_16    equ     $000800
SAMP_RATE_8     equ     $000000
STEREO          equ     $000400
DATA_8LIN       equ     $200300
DATA_8A         equ     $200200
DATA_8U         equ     $200100
DATA_16         equ     $200000
IMMED_3STATE    equ     $800000
XTAL2_SELECT    equ     $200000
BITS_64         equ     $000000
BITS_128        equ     $040000
BITS_256        equ     $080000
CODEC_MASTER    equ     $020000
CODEC_TX_OFF    equ     $010000

CTRL_WD_12      equ     NO_PREAMP+HI_PASS_FILT+SAMP_RATE_48+STEREO+DATA_16   ;CLB=0
CTRL_WD_34      equ     IMMED_3STATE+XTAL2_SELECT+BITS_64+CODEC_MASTER
CTRL_WD_56      equ     $000000
CTRL_WD_78      equ     $000000

HEADPHONE_EN    equ     $800000
LINEOUT_EN      equ     $400000
LEFT_ATTN       equ     $010000 ;63*LEFT_ATTN   = -94.5 dB, 1.5 dB steps
SPEAKER_EN      equ     $004000
RIGHT_ATTN      equ     $000100 ;63*RIGHT_ATTN  = -94.5 dB, 1.5 dB steps
MIC_IN_SELECT   equ     $100000
LEFT_GAIN       equ     $010000 ;15*LEFT_GAIN    = 22.5 dB, 1.5 dB steps
MONITOR_ATTN    equ     $001000 ;15*MONITOR_ATTN = mute,    6   dB steps
RIGHT_GAIN      equ     $000100 ;15*RIGHT_GAIN   = 22.5 dB, 1.5 dB steps
OUTPUT_SET      equ     HEADPHONE_EN+LINEOUT_EN+(LEFT_ATTN*2)
INPUT_SET       equ     MIC_IN_SELECT+(15*MONITOR_ATTN)+(RIGHT_ATTN*2)

;---DSP56002 on-chip peripheral addresses
IPR             equ     $FFFF   ;Interrupt Priority Register
BCR             equ     $FFFE   ;Bus Control Register
PLL             equ     $FFFD   ;PLL Control Register
SSIDR           equ     $FFEF   ;SSI Data Register
SSISR           equ     $FFEE   ;SSI Status Register
CRB             equ     $FFED   ;SSI Control Register B
CRA             equ     $FFEC   ;SSI Control Register A
PCD             equ     $FFE5   ;Port C Data Register
PBD             equ     $FFE4   ;Port B Data Register
PCDDR           equ     $FFE3   ;Port C Data Direction Register
PBDDR           equ     $FFE2   ;Port B Data Direction Register
PCC             equ     $FFE1   ;Port C Control Register
PBC             equ     $FFE0   ;Port B Control Register
SCR             equ     $FFF0   ;SCI interface control register
SCCR            equ     $FFF2   ;SCI control register

START           equ     $40

;----------------------------------------------------------------------------
	org     p:0
        jmp     StartHi

;----------------------------------------------------------------------------
	org     p:$C
	jsr     ssi_rx_isr              ;SSI RX
	jsr     ssi_rx_isr              ;SSI RX w/Exception
	jsr     ssi_tx_isr              ;SSI TX
	jsr     ssi_tx_isr              ;SSI TX w/Exception

;****************************************************************************

        org     p:START

;************************** SSI receive ISR**************************
ssi_rx_isr
	move    r0,x:(r6)+              ; Save r0 to the stack.
	move    m0,x:(r6)+              ; Save m0 to the stack.
	move    #3,m0                   ; Modulo 4 buffer.
        move    y:RX_PTR,r0             ; Load the pointer to the rx buffer.
	jclr    #3,x:SSISR,next_rx      ; If not fr. syc, jump to receive data.
	move    #RX_BUFF_BASE,r0        ; If frame sync, reset base pointer.
	nop
next_rx
        movep   x:SSIDR,y:(r0)+         ; Read out received data to buffer.
        move    r0,y:RX_PTR             ; Update rx buffer pointer.
	move    x:-(r6),m0              ; Restore m0.
	move    x:-(r6),r0              ; Restore r0.
        rti


; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;****************************************************************************


TONE_OUTPUT     EQU     HEADPHONE_EN+LINEOUT_EN+(0*LEFT_ATTN)+(0*RIGHT_ATTN)
TONE_INPUT      EQU     MIC_IN_SELECT+(15*MONITOR_ATTN)


;----------------------------------------------------------------------------
       org     x:0

;----------------------------------------------------------------------------
stack           DS      64      ; 64 Words reserved for stack (pointer r6)
;----------------------------------------------------------------------------
;  Anzeigen
LED_tmr         DS      1       ; main-Timer f. LEDs (IRQ)
LED_flag        DS      1       ; Flags for LED (IRQ->main prg.)
LED_sig         DS      1       ; timer f. Signal-LED (green)
LED_ov          DS      1       ; timer f. Over-LED (rot)

;----------------------------------------------------------------------------
sw1_buf         DS      1       ; Fx Type Switch
sw2_buf         DS      1       ; Parameter 1 Switch
sw3_buf         DS      1       ; Parameter 2 Switch
Sw_changed      DS      1       ; Switch changed flag:  bit1-FxType changed
                                ;                       bit2-Param changed
;----------------------------------------------------------------------------
; Quantizer
acount          DS      1       ; alias counter f. Fs-divider
astore          DS      2       ; Alias Sample Buffer (L-R)

;----------------------------------------------------------------------------
; DelayLine
DLbuf           EQU     $1000   ; buffer start adr.
DL_lenght       DS      1       ; Delaylenght in Mem (=DelayTaps/1.5)
                                ; data packed (3:2)
DL_pointer      DS      1       ; Pointer to input data
DL_flag         DS      1       ; Flag (bit0) shows, where next data
                                ; (packed) 0=normal (bit 8-23)
                                ; 1= bit 0-7 + next word bit 0-7
;----------------------------------------------------------------------------
; Reverse
rvs_flag        DS      1       ; flag, mult. by  -1 oder +1
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
; NextFX

;----------------------------------------------------------------------------
        org x:$600  ; make space for rest of code
                    ; (P: = X: $200 to max. $5ff)

;----------------------------------------------------------------------------


;instates        DSM     4096 ; Ringbuffer L-R-Format
;----------------------------------------------------------------------------

;states_d        ; == delay_hp same buffer for delay and dez.filter
;delay_hp        DSM     ndel    ; Phasenkorrektur-delay
_endx

;******************************************************************************

       org     y:0

;        BADDR   M,dtaps  ; Tabelle fr Dezimations-Koeffs
;coef_i
;coef_d

;        include 'dez_int.asm'  ; die Koeffs sind schon mit faktor 16
                                ; gewichtet, es muss also einmal das
                                ; Signal durch 16 dividiert werden !!!

;----------------------------------------------------------------------------
RX_BUFF_BASE    equ     *
RX_data_1_2     ds      1       ;data time slot 1/2 for RX ISR
RX_data_3_4     ds      1       ;data time slot 3/4 for RX ISR
RX_data_5_6     ds      1       ;data time slot 5/6 for RX ISR
RX_data_7_8     ds      1       ;data time slot 7/8 for RX ISR

TX_BUFF_BASE    equ     *
TX_data_1_2     ds      1       ;data time slot 1/2 for TX ISR
TX_data_3_4     ds      1       ;data time slot 3/4 for TX ISR
TX_data_5_6     ds      1       ;data time slot 5/6 for TX ISR
TX_data_7_8     ds      1       ;data time slot 7/8 for TX ISR

RX_PTR          ds      1       ; Pointer for rx buffer
TX_PTR          ds      1       ; Pointer for tx buffer

;****************************************************************************


;        BADDR   M,ktaps    ; Koeffs for Kernfilter
;coef_k 

;        include 'kern_150.asm'

;----------------------------------------------------------------------------
;   LIMITER - STEUERWERT - TABELLE

;ktab    DUPF    NUM,0,4095              ; Tabelle mit 4096 Werten
;        DC      4095.0/(7.0*NUM+4095.0) ; for yt(n)=4095/(7n+4095)
;        ENDM                            ; ergibt 12 bit Genauigkeit

;----------------------------------------------------------------------------

_endy  ; Endemarkierung (Y-Bereich) fuer den Debugger
;******************************************************************************

	org     p:

;****************************************************************************
;
;              ALL SIGNAL PROCESSING IS FOLLOWING HERE
;
;****************************************************************************

;**************** new SSI TRANSMIT ISR (with processing) ********************
ssi_tx2
	move    r0,x:(r6)+              ; Save r0 to the stack.
	move    m0,x:(r6)+              ; Save m0 to the stack.
	move    #3,m0                   ; Modulus 4 buffer.
        move    y:TX_PTR,r0             ; Load the pointer to the tx buffer.
        jclr    #2,x:SSISR,next_t2      ; If not frame sync, jump to transmit data.
	move    #TX_BUFF_BASE+1,r0      ; If frame sync, reset pointer.
        nop
        movep   y:(r0)+,x:SSIDR         ; SSI transfer data register.
        move    r0,y:TX_PTR             ; Update tx buffer pointer.
        move    x:-(r6),m0              ; restore m0 from stack
        move    x:-(r6),r0              ; restore r0

        andi    #$FC,mr     ; enable interrupts - RX_ISR (4x) und diese ISR
        ; (3x) sollen diese laufende ISR (mit der Sig.verarb.) unterbrechen

; im Hauptprogramm drfen a, x0 und y0 verndert werden
; deshalb vor der Signalverarbeitung sichern

        move    a0,x:(r6)+              ; save a0 to stack
        move    a1,x:(r6)+              ; save a1 to stack
        move    a2,x:(r6)+              ; save a2 to stack
        move    x0,x:(r6)+              ; save x0 to stack
        move    y0,x:(r6)+              ; save y0 to stack


;----------------------------------------------------------------------------
        jsr     process         ; Pegelanzeige aktualisieren
                                ; und je nach Schalterstellung zum
                                ; Fx processing verzweigen
;----------------------------------------------------------------------------

; ... und wieder restaurieren
        move    x:-(r6),y0              ; restore y0 from stack
        move    x:-(r6),x0              ; restore x0 from stack
        move    x:-(r6),a2              ; restore a2 from stack
        move    x:-(r6),a1              ; restore a1 from stack
        move    x:-(r6),a0              ; restore a0 from stack

        rti
next_t2
        movep   y:(r0)+,x:SSIDR         ; SSI transfer data register.
        move    r0,y:TX_PTR             ; Update tx buffer pointer.
	move    x:-(r6),m0              ; Restore m0.
	move    x:-(r6),r0              ; Restore r0.
	rti

;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
process         ; LED Pegelanzeige grn=Signal rot=Over
                ;     Anzeigen aktualisieren

        move    y:RX_BUFF_BASE,x0       ; LINKER Eingang
        move    y:RX_BUFF_BASE+1,x1     ; RECHTER Eingang
        clr     a
        move    x:LED_tmr,a0            ; Zentral-Timer f. LEDs
        dec     a
        jpl     lflg
        move    #>LED_time,a0
lflg    move    a0,x:LED_tmr

        move    #>$080000,a             ; Signal LED
        cmpm    x0,a
        jpl     we1
        bset    #0,x:LED_flag
we1     cmpm    x1,a
        jpl     weiter
        bset    #0,x:LED_flag
weiter
        move    #$7ffe00,a              ; OVER-LED
        cmpm    x0,a
        jpl     wei1
        bset    #1,x:LED_flag
wei1    cmpm    x1,a
        jpl     weit
        bset    #1,x:LED_flag

; Sprung in die Processing routine ber Tabelle abh. von FxTypeSwitch 
weit    btst    #1,x:Sw_changed
        jcs     jinit
        move    #jtab1,r7       ; Sprungtabelle fr FxProcessing
        move    x:sw1_buf,n7    ; offset je nach Schalterstellung
        nop
        jmp     (r7+n7)         ; Fx routine aufrufen

jinit   move    #jtab2,r7       ; Sprungtab. fr Initialisierung des neuen
                                ; FxType nach Schalternderung
        move    x:sw1_buf,n7    ; offset je nach Schalterstellung
        bclr    #1,x:Sw_changed ; Flag lschen
        jmp     (r7+n7)         ; InitFx routine aufrufen

;       Der rts-Befehl zum zurckkehren in die isr mu
;       in der Processingroutine stehen !

;----------------------------------------------------------------------------
;  Signalverarbeitung ist hier
;  beim Aufruf stehen die Eingangswerte noch in x0,x1
;----------------------------------------------------------------------------
Quantizer       ; Aliasing Effekt + Downscale Effekt
                ; Param1: Depth 0..15 (Fs=48kHz..3kHz)
                ; Param2: BitResolution 0..15 (16bit..1Bit)

        clr     a

        move    x:sw2_buf,a0    ; wert von switch2 (Quantisation)
        inc     a               ; um 1 erhhen (wert: 1..16)
        move    a0,y0           ; = anzahl der schleifendurchlufe

        move    x:acount,a0     ; zhler f. alias-effect
        dec     a
        move    a0,x:acount
        jne     qntz

        move    x:sw3_buf,a0
        inc     a
        move    a0,x:acount
        move    x0,x:astore     ; LINKEN Eingang in alias buffer speichern
        move    x1,x:astore+1   ; RECHTEN Eingang    ''

qntz    move    x:astore,a      ; werte aus alias buffer holen
        move    x:astore+1,b

        do      y0,qlp1_end
        asr     a
        asr     b
qlp1_end
        do      #7,qlp2_end
        asr     a
        asr     b
qlp2_end
        rnd     a
        rnd     b
        do      y0,qlp3_end
        asl     a
        asl     b
qlp3_end
        do      #7,qlp4_end
        asl     a
        asl     b
qlp4_end

        move    a,y:TX_BUFF_BASE     ; LINKER Ausgang
        move    b,y:TX_BUFF_BASE+1   ; RECHTER Ausgang
        rts
;----------------------------------------------------------------------------
;DelayLine      ; Line Delay  0% Feedback, 0-400ms L->L, R->R
                ; Param1 : Time Coarse 24.0-360 ms    (0-18432 step 1152)
                ; Param2 : Time Fine    1.5- 22.5 ms  (0- 1080 step   72) 
                ; Mem: L->X , R->Y $1000-$3fff   (12240 Words je X,Y)
                ; gepackt: 3 16bit-Worte in 2 24bit Worten

DLInit
        move    x1,y0           ; R-Input retten
        move    #DLbuf,r0       ; r0 zeigt auf den AUSgangspuffer !!!
        move    #DLbuf,r1       ; r1 zeigt auf den EINgangspuffer  !!!
        move    #0,n0
        move    #0,n1
        move    #$FFFF,m0
        move    #$FFFF,m1 ; lin. Adressierung kein Ringb. mglich
        move    #0,y1
        move    y1,x:DL_pointer ; pointer rcksetzen
        move    y1,x:DL_flag    ; flag lschen
        bset    #1,x:DL_flag    ; am Anfang nur eimnal unpacked

DLProcess
        move    x1,y0
        move    x:sw2_buf,x1    ; coarse time param (0..15)
        move    #>1152,y1       ; coarse step
        mpy     x1,y1,a         ; DL coarse Lnge in a0
        move    x:sw3_buf,x1    ; fine time param (0..15)
        move    #>72,y1         ; fine step
        mac     x1,y1,a         ; in a0 steht jetzt die DL-Lnge in taps
        asr     a
        move    a0,x1
        move    #$555556,y1
        mpy     x1,y1,a         ; dividieren durch 1.5 (= *0.66666667)
                                ; in a1 steht jetzt die DL-Lnge im Mem
        move    a1,x:DL_lenght  ; speichern
        move    x:DL_pointer,n0 ; Ausgangspointer laden
        move    x:DL_pointer,x1
        add     x1,a           ; Eingangspointer berechnen
        move    #>$2FFE,x1
        cmpm    x1,a
        jmi     DL_w1
        move    #>$2FFE,x1
        sub     x1,a
DL_w1   move    a1,n1           ; n1 : Eingangsoffset !!!!!!!
        clr     a
        btst    #0,x:DL_flag    ; 0=normal (bit 8-23), 1=gepackt (bit 0-7)
                                ;                   + nchstes Wort bit 0-7
        jcs     packed

        move    x:(r1+n1),a1    ; die 8 msb vom vorhergehenden wert retten
        move    #>$0000FF,y1
        and     y1,a1        ; 8 bit isolieren
        or      x0,a1        ; die aktuellen 16 bit dazu
        move    a1,x:(r1+n1)    ; speichern

        move    y:(r1+n1),a1    ; dasselbe nochmal f. R
        move    #>$0000FF,y1
        and     y1,a1
        or      y0,a1           ; die aktuellen 16 bit fr R dazu
        move    a1,y:(r1+n1)    ; speichern

        move    x:(r0+n0),x0    ; Ausgang L (unpacked)
        move    y:(r0+n0),y0    ;   ''    R

        move    x0,y:TX_BUFF_BASE     ; LINKER Ausgang
        move    y0,y:TX_BUFF_BASE+1   ; RECHTER Ausgang

        btst    #1,x:DL_flag    ; schon eimnal unpacked ?
        jcc     DL_unp
        bset    #0,x:DL_flag    ; flag setzen, nchster wert ist packed!
        bclr    #1,x:DL_flag    ; das 'schon 1mal unpacked-flag' lschen
        rts

DL_unp  bset    #1,x:DL_flag    ; einmal unpacked, danach noch eimnal
        jmp     DL_pnt          ; pointer erhhen

packed  move    x:(r1+n1),a     ; die 16 msb sind schon belegt
        move    #$FFFF00,y1
        and     y1,a1     ; die 16 msb isolieren
        move    a1,y1
        move    x0,b            ; akt. L-Wert holen
        rep     #8              ; 8 mal rechts schieben
        asr     b
        move    b,x1            ; die hohen 8 bit werden noch gebraucht
        move    #>$0000FF,x0
        and     x0,b1          ; die 8 lsb isolieren
        or      y1,b1          ; die neuen 8lsb zu den vorhandenen 16 msb
        move    b1,x:(r1+n1)   ; speichern
        clr     a  (r1)+       ; die higher 8bit sind die lsb an der
        nop                    ; folgenden Adresse
        move    x:(r1+n1),a
        move    #$FFFF00,y1
        and     y1,a1
        move    a1,y1
        move    x1,b
        rep     #8              ; 8 mal rechts schieben
        asr     b
        move    #>$0000FF,x1
        and     x1,b1
        or      y1,b1           ; die neuen 8lsb zu den vorhandenen 16 msb
        move    b1,x:(r1+n1)    ; speichern
        clr     a  (r1)-
        nop
                   ; jetzt packed fr den R-Kanal
        move    y:(r1+n1),a     ; die 16 msb sind schon belegt
        move    #$FFFF00,y1
        and     y1,a1     ; die 16 msb isolieren
        move    a1,y1
        move    y0,b            ; akzt. R-Wert holen
        rep     #8              ; 8 mal rechts schieben
        asr     b
        move    b,x1            ; die hohen 8 bit werden noch gebraucht
        move    #>$0000FF,y0
        and     y0,b1
        or      y1,b1          ; die neuen 8lsb zu den vorhandenen 16 msb
        move    b1,y:(r1+n1)   ; speichern
        clr     a  (r1)+       ; die nchsten 8bit sind die lsb an der
        nop                    ; folgenden Adresse
        move    y:(r1+n1),a
        move    #$FFFF00,y1
        and     y1,a1
        move    a1,y1
        move    x1,b
        rep     #8              ; 8 mal rechts schieben
        asr     b
        move    #>$0000FF,x1
        and     x1,b1
        or      y1,b1           ; die neuen 8lsb zu den vorhandenen 16 msb
        move    b1,y:(r1+n1)    ; speichern
        clr     a  (r1)-        ; r0 auf Anfangswert zurcksetzen

                      ; jetzt die packed daten laden
        clr     b  (r0)+       ; r1 fr das high byte der packed daten
        nop                    ; erhhen
        move    x:(r0+n0),a    ; 8lsb laden (= high byte der packed daten)
        move    #>$0000FF,x0
        and     x0,a1          ; 8lsb isolieren
        rep     #7
        asl     a              ; 8 bit links shieben
        asl     a  (r0)-       ; r1 auf Anfangswert zurck fr das low byte
        move    a1,x1
        move    x:(r0+n0),a 
        move    #>$0000FF,x0
        and     x0,a1
        or      x1,a1  (r0)+   ; zum high byte dazu, r1 fr high byte erh.
        rep     #8
        asl     a
        move    a1,x0

        move    y:(r0+n0),a    ; dasselbe f. Rechts
        move    #>$0000FF,y0
        and     y0,a1
        rep     #7
        asl     a
        asl     a  (r0)-       ; r1 wieder zurck
        move    a1,x1
        move    y:(r0+n0),a
        move    #>$0000FF,y0
        and     y0,a1
        or      x1,a1
        rep     #8
        asl     a
        move    a1,y0

        move    x0,y:TX_BUFF_BASE     ; LINKER Ausgang
        move    y0,y:TX_BUFF_BASE+1   ; RECHTER Ausgang

        bclr    #0,x:DL_flag    ; flag lschen, nchster wert ist unpacked!!

DL_pnt  move    x:DL_pointer,a  ; neuen Pointer berechnen
        move    #>1,x1
        add     x1,a           ; eins dazu
        move    #>$2FFE,x1
        cmpm    x1,a
        jmi     DL_w2
        move    #>$2FFE,x1
        sub     x1,a
DL_w2   move    a1,x:DL_pointer ; neuer pointer
        rts
;----------------------------------------------------------------------------
Reverse         ; Reverse Spectrum
                ; No parameters
        bchg    #1,x:rvs_flag
        btst    #1,x:rvs_flag
        jcc     rvs_w2
        move    #-1.0,y0
        btst    #0,x:rvs_flag
        jcc     rvs_w1
        move    #1.0,y0
rvs_w1  bchg    #0,x:rvs_flag
rvs_w2  mpy     x0,y0,a
        mpy     x1,y0,b
        move    a,y:TX_BUFF_BASE     ; LINKER Ausgang
        move    b,y:TX_BUFF_BASE+1   ; RECHTER Ausgang

        rts

;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
Nopi    ; test - Bypass
        move    x0,y:TX_BUFF_BASE     ; LINKER Ausgang
        move    x1,y:TX_BUFF_BASE+1   ; RECHTER Ausgang

        rts

;----------------------------------------------------------------------------
Q    ; test - mute outputs
        rts

;----------------------------------------------------------------------------
dsv_end    ; Endemarkierung zum Debuggen

;****************************************************************************
;               Ende der Signalverarbeitung
;****************************************************************************



        org     p:$0200  ; hier ist auch X-Memory !!!
                        ; p:=x: $200 - $5ff ist fr Programmcode reserviert

;---------------------------------------------------------------------------
;
;      Main codec initialization program
;
;---------------------------------------------------------------------------

StartHi movep   #$26100F,x:PLL          ;set PLL for MPY of 16x ==> 64MHz!!!
	movep   #$0000,x:BCR            ; number of wait states
	ori     #3,mr                   ;disable interrupts
	movec   #0,sp
	move    #0,omr                  ; single chip mode

        move    #stack,r6               ; initialize stack pointer r6
                                        ; (stack: X, 64 words free)
	move    #-1,m6                  ; linear addressing
        move    #RX_BUFF_BASE,y0
        move    y0,y:RX_PTR             ; Initialize the rx pointer
        move    #TX_BUFF_BASE,y0
        move    y0,y:TX_PTR             ; Initialize the tx pointer
        jsr     codec_init              ; CODEC am SSI

; The following line of code jumps to "main" which is the location for
; the main code that will run after the codec is initialized.

	jmp	main


;***************************************************************************
;***** 			initialize the CS4215 codec                    *****
;***************************************************************************
; headphones and line out, and set up for no gain or attenuation, and no 
; monitor feedback.
;***************************************************************************
;***************************************************************************
;
;      initialize ssi -- fsync and sclk ==> outputs
;
codec_init
	movep   #$0000,x:PCC    ;  turn off ssi port 
        movep   #$4307,x:CRA    ;  64MHz/32 = 2.0MHz SCLK, WL=16 bits, 4W/F
	movep   #$FB30,x:CRB    ; RIE,TIE,RE,TE, NTWK, SYN, FSR/RSR->bit
        movep   #$14,x:PCDDR    ; setup pc2 and pc4 as outputs
	movep   #$0,x:PCD       ; D/C~ and RESET~ = 0 ==> control mode
				;----reset delay for codec ----
	do      #500,_delay_loop
        rep     #1600           ; 100 us delay  (64MHz)!!!
	nop
_delay_loop
	bset    #4,x:PCD        ; RESET~ = 1
        movep   #$3000,x:IPR    ; set interrupt priority level
        andi    #$FC,mr         ; enable interrupts

;*****************************************************************************
; The following data sets up the CS4215 control mode data:
;    (CTS = Control Time Slot, U/LN = upper/lower Nibble)
;
;       +------ CTS1-UN:   0      0      1     MLB      0 0 0 0 
;       |+----- CTS1-LN:  OLB    CLB     X      X       0 0 0 0 
;       ||+---- CTS2-UN:  HPF     X     DFR2   DFR1     0 0 1 0
;       |||+--- CTS2-LN:  DFR0   ST     DF1    DF0      1 1 0 0
; x0 = $002Cxx
;
;       +------ CTS3-UN:  ITS    MCK2   MCK1   MCK0     1 0 0 0
;       |+----- CTS3-LN:  BSEL1  BSEL0  XCLK   XEN      1 0 0 0 
;       ||+---- CTS4-UN:  TEST   TEST   TEST   TEST     (TEST MUST BE 0)
;       |||+--- CTS4-LN:  TEST   TEST   ENL    DAD      0 0 0 0 
; x0 = $8800xx
;*****************************************************************************

;--- set up buffer with control mode data
        move    #CTRL_WD_12,y0
        move            y0,y:TX_BUFF_BASE
        move    #CTRL_WD_34,y0
        move            y0,y:TX_BUFF_BASE+1
        move    #CTRL_WD_56,y0
        move            y0,y:TX_BUFF_BASE+2
        move    #CTRL_WD_78,y0
        move            y0,y:TX_BUFF_BASE+3

        movep   #$01E8,x:PCC    ; Turn on ssi port

;
; CLB == 0
;
	jclr    #3,x:SSISR,*            ; wait until rx frame bit==1
	jset    #3,x:SSISR,*            ; wait until rx frame bit==0
	jclr    #3,x:SSISR,*            ; wait until rx frame bit==1
wait1   move    y:RX_BUFF_BASE,y0
        jset    #18,y0,wait1            ; loop until CLB set

;
; CLB == 1
;
        bset    #18,y:TX_BUFF_BASE      ;set CLB
	do      #4,_init_loopB
	jclr    #2,x:SSISR,*            ; wait until tx frame bit==1
	jset    #2,x:SSISR,*            ; wait until tx frame bit==0
_init_loopB
	movep   #0,x:PCC                ;disable, reset SSI

;*****************************************************************************
;    now CLB should be 1 -- re-program fsync and sclk direction (i/p) -- also,
;  circular buffer pointers for echoing data r0=current, r1=old data to send
;  1 frame later
;
        movep   #$4307,x:CRA    ; 16bits,4 word/frame, /32=2.0 MHz
	movep   #$FB00,x:CRB    ; rcv,xmt & int ena,netwk,syn,sclk==i/p,msb 1st
        movep   #$14,x:PCD      ; D/C~ pin = 1  ==> data mode
        movep   #$01E8,x:PCC    ;  turn on ssi port
	rts
; ***************************************************************************

;
;************************** SSI TRANSMIT ISR **************************
;  - diese tx_isr ist nur fr die init-phase ! (vektor wird verbogen)
ssi_tx_isr
	move    r0,x:(r6)+              ; Save r0 to the stack.
	move    m0,x:(r6)+              ; Save m0 to the stack.
	move    #3,m0                   ; Modulus 4 buffer.
        move    y:TX_PTR,r0             ; Load the pointer to the tx buffer.
	jclr    #2,x:SSISR,next_tx      ; If not frame sync, jump to transmit data.
	move    #TX_BUFF_BASE+1,r0      ; If frame sync, reset pointer.
        nop
next_tx
        movep   y:(r0)+,x:SSIDR         ; SSI transfer data register.
        move    r0,y:TX_PTR             ; Update tx buffer pointer.
	move    x:-(r6),m0              ; Restore m0.
	move    x:-(r6),r0              ; Restore r0.
	rti

;----------------------------------------------------------------------------
	

;----------------------------------------------------------------------------
main

; Initialize global Parameters

;----------------------------LEDs--------------------------------
        move    #>0,x0
        move    x0,x:LED_flag   ; Flags f. Anzeigen initialisieren
        movep   #$0,x:PBC       ; Port B as GPIO
        movep   #%0011000000000000,x:PBDDR  ; bits 12,13 sind fuer die LEDs
                                            ; bits 0..11 fr die Schalter
        move    #>LED_time,a
        move    a,x:LED_tmr

        move    #$ffff,m7       ; r7+n7 sind der pointer auf die Sprungtab.
        move    #$0,n7

;        move    #instates,r1
;        move    #$0fff,m1
;        move    #0,n1

        move    #0,x0          ; reset switch buffers
        move    x0,x:sw1_buf
        move    x0,x:sw2_buf
        move    x0,x:sw3_buf

                                ; ! r6 ist schon (oben) als Stackpointer
                                ; initialisiert (fr die ISRs)
;----------------------------------------------------------------------------

; Initialize Codec In/Out Parameters
   move    #TONE_OUTPUT,y0         ; headphones, line out, mute spkr, no attn.
   move            y0,y:TX_BUFF_BASE+2
   move    #TONE_INPUT,y0          ; no input gain, monitor mute
   move            y0,y:TX_BUFF_BASE+3
;----------------------------------------------------------------------------
;
;        do      #1000,pause1   ; pause fr interrupt
;        rep     #25
;        nop                     ; bevor die tx-vektoren verbogen werden
;pause1

; Re- program Interrupt vectors to ISR with processing
        ori     #$03,mr         ; disable interrupts
        move    #>ssi_tx2,x0
        movem   x0,p:$11  ; new ssi_tx-vector
        movem   x0,p:$13  ; new ssi_tx/e vector
        andi    #$fc,mr         ; enable interrupts



;----------------------------------------------------------------------------

; Main Loop   ( Hauptprogramm ist reserviert fr Anzeige & Bedienung)
;             ( weil niedrigste Prioritt und asynchron zu Fs )
;          - A,X0,Y0 - register drfen veraendert werden


loop_1 

        btst    #0,x:LED_flag           ; Signal-LED
        jcc     w1
        bclr    #0,x:LED_flag
        bset    #12,x:PBD
        move    x:LED_tmr,a0
        inc     a
        move    a0,x:LED_sig
        jmp     w2
w1      move    x:LED_tmr,x0
        move    x:LED_sig,a
        cmp     x0,a
        jne     w2
        bclr    #12,x:PBD
w2
        btst    #1,x:LED_flag           ; OVER-LED
        jcc     w5
        bclr    #1,x:LED_flag
        bset    #13,x:PBD
        move    x:LED_tmr,a0
        inc     a
        move    a0,x:LED_ov
        jmp     w6
w5      move    x:LED_tmr,x0
        move    x:LED_ov,a
        cmp     x0,a
        jne     w6
        bclr    #13,x:PBD
w6
        ;        jmp     loop_1  ; jmp back without swtch for test !!!


; ***************************************************************************
; *             Hauptprg. f. Schalterabfrage                                *
; ***************************************************************************
; *
; *     Schalterabfrage : 4 bits = 16 Stellungen
; *     FxTypeSwitch    : PB 0..3
; *     Param 1         : PB 4..7
; *     Param 2         : PB 8..11
;
        
        clr     a
        movep   x:PBD,a1        ; Schalter
        move    #>15,x0
        and     x0,a1           ; FxType: Bits 0..3 sind interessant
        move    x:sw1_buf,x0    ; Puffer f. FxTypeSwitch
        move    a,x:sw1_buf     ; neuer Schalterwert in Puffer
        cmp     x0,a            ; Vergleich : Schalter geaendert ?
        jeq     Sw0             ; wenn ungleich - Schalter geaendert
        bset    #1,x:Sw_changed ; bit1: Type changed flag

Sw0     movep   x:PBD,a         ; Schalter
        rep     #4
        lsr     a               ; vier Bits nach rechts
        move    #>15,x0
        and     x0,a1           ; Param1: Bits 4..7 sind interessant
        move    x:sw2_buf,x0    ; Puffer f. Param1
        cmp     x0,a            ; Vergleich : Schalter geaendert ?
        jeq     Sw1             ; wenn ungleich - Schalter geaendert
        move    a,x:sw2_buf
        bset    #2,x:Sw_changed ; bit2: Param changed flag

Sw1     movep   x:PBD,a         ; Schalter
        rep     #8
        lsr     a               ; acht Bits nach rechts
        move    #>15,x0
        and     x0,a1           ; Param2: Bits 8..11 sind interessant
        move    x:sw3_buf,x0    ; Puffer f. Param2
        cmp     x0,a            ; Vergleich : Schalter geaendert ?
        jeq     Sw2             ; wenn ungleich - Schalter geaendert
        move    a,x:sw3_buf
        bset    #2,x:Sw_changed ; bit2: Param changed flag


Sw2     jmp     loop_1                  ; Loop back.

;       Ende des Hauptprogs
; ***************************************************************************

jtab1           JMP Quantizer    ; Sprungtabelle fr
                JMP DLProcess    ; die 16 FxTypes
                JMP Nopi         ; normale routine
                JMP Reverse
                JMP Q
                JMP Q
                JMP Q
                JMP Q
                JMP Q
                JMP Q
                JMP Q
                JMP Q
                JMP Q
                JMP Q
                JMP Q
                JMP Q

jtab2           JMP Quantizer    ; Sprungtabelle fr 
                JMP DLInit       ; die Initialisierungsroutinen
                JMP Nopi         ; der FxTypes
                JMP Reverse
                JMP Q
                JMP Q
                JMP Q
                JMP Q
                JMP Q
                JMP Q
                JMP Q
                JMP Q
                JMP Q
                JMP Q
                JMP Q
                JMP Q
;----------------------------------------------------------------------------



;**************************************************************************
;   FLASH.ASM - routine to make user code bootable from a flash eeprom
;
;   3 Nov. 1994 : version 1.0   rlr/ljd
;  13 Apr. 1995 :         1.1   rlr - force write to low byte (enable_prot)
;  25 Oct. 1995 :         1.2   rlr - correct conversion to sector count
;
;   9 Sep. 1996 :         1.99  spf - check how the cow gets mad. ? 
;   6 Jan. 1997 :         2.00  spf - adaption to EvilFx Project
;
;   copyright (C) Motorola 1994
;
;
;   -- feste Adressbereiche fr EvilFx werden gespeichert !!!
;
;**************************************************************************

;BCR             equ     $FFFE           ;Bus Control Register
PLL_CTRL        equ     $FFFD           ;PLL Control Register
begin_address   equ     $3F00           ;The loader will be copied to p:$3f00

        org     p:$3E00

save_main
;---------------------------------------------------------------------------
;  An external boot-ROM must have the lowest byte of data at the lowest
;  address.  (i.e.,  ROM Address $C000 --- P:0000 low    byte
;                    ROM Address $C001 --- P:0000 middle byte
;                    ROM Address $C002 --- P:0000 high   byte
;                    ROM Address $C003 --- P:0001 low    byte ....)
;---------------------------------------------------------------------------

        movep   #$261009,x:PLL_CTRL     ;set PLL for MF=10, 40 MHz VCO out
        movep   #$B0B0,x:BCR            ;at 40MHz, T=25ns, 11 wait states in P:


;------ move the bootstrap loader into FLASH at P:$C000
;        move    r1,P:boot+$E            ;modify boot for length
	move	#boot,r0		;first DSP address to save
	move	#$C000,r2		;flash starting address
	jsr	WR_FLASH		;save 64 words


;        move    r1,A                    ;convert word cnt to sector cnt.
;        rep     #6
;        asr     A 
;        move    A1,r1
;        nop
;        move    (r1)+

; --------------------- P: --------------------------------------------------
; ************* nur der interne P: Bereich ist gemeint !! *******************
;
        move    #$08,r1      ; Anzahl der 64-Wort-Bloecke
        move    #$D000,r2               ;address of initial FLASH address
        move    #$0000,r0               ;r0 points to input code

 	do	r1,loop			;write sectors
	jsr	WR_FLASH
loop
; ----------------------- X: ------------------------------------------------
; ************* incl. ext. X-Bereich, == gleichz. ext. P-Bereich !! *********
;
        move    #$18,r1            ; Anzahl der 64-Wort-Bloecke
        move    #$E000,r2               ;address of initial FLASH address
        move    #$0000,r0               ;r0 points to input X: ram

        do      r1,loopx                 ;write sectors
        jsr     WR_FLA_X
loopx
; ---------------------------- Y: -------------------------------------------
        move    #$51,r1            ; Anzahl der 64-Wort-Bloecke
        move    #$8000,r2               ;address of initial FLASH address
        move    #$0000,r0               ;r0 points to input Y: ram

        do      r1,loopy                 ;write sectors
        jsr     WR_FLA_Y
loopy
; ---------------------------------------------------------------------------

        movep  #$0000,x:BCR
        debug
        jmp    *


;************************************************************************
;   WR_FLASH             !!! P-RAM !!!
;   This routine writes 64 (24-bit) words from DSP RAM to FLASH EEPROM
; 
;	enter with:   	r2 pointing to starting EEPROM Byte Address
;      			r0 pointing to first DSP word to save
;
;-------------------------------------------------------------------------
;   first, move 64 bytes - this includes 21 words + low byte of 22nd
;-------------------------------------------------------------------------
WR_FLASH
        jsr     disable_protect

        do      #21,move_codeA          ;move 63 bytes
        move    P:(r0)+,A               ;get 3 byte word
        move            A1,x:(r2)+	;move low  byte to FLASH EEPROM
        rep     #8
        lsr     A       
        move            A1,x:(r2)+      ;move mid  byte to FLASH EEPROM
        rep     #8
        lsr     A       
        move            A1,x:(r2)+      ;move high byte
move_codeA
					;*** 64th byte
        move    P:(r0)+,A               ;get next 3 byte word
        move            A1,x:(r2)+      ;move low  byte

        jsr     delay_200u              ;wait 200 microseconds to enter 
                                        ;...program cycle
        jsr     program_complete        ;wait for programming cycle to 
                                        ;...finish
;-------------------------------------------------------------------------
;   then, move another 64 bytes - mid and high bytes of 22nd word followed
;                                 by 20 words and then the low and mid
;				  bytes of the next word
;-------------------------------------------------------------------------
        jsr     disable_protect

        rep     #8			
        lsr     A       
        move            A1,x:(r2)+      ;move mid  byte to FLASH EEPROM
        rep     #8
        lsr     A       
        move            A1,x:(r2)+      ;move high byte


        do      #20,move_codeB          ;move 60 bytes
        move    P:(r0)+,A		;get 3 byte word
        move            A1,x:(r2)+	;move low  byte to FLASH EEPROM
        rep     #8
        lsr     A       
        move            A1,x:(r2)+      ;move mid  byte to FLASH EEPROM
        rep     #8
        lsr     A       
        move            A1,x:(r2)+      ;move high byte
move_codeB
					;*** 63rd byte ***
        move    P:(r0)+,A		;get next 3 byte word
        move            A1,x:(r2)+      ;move low byte
        rep     #8
        lsr     A       		;*** 64th byte ***
        move            A1,x:(r2)+      ;move mid  byte to FLASH EEPROM

        jsr     delay_200u              ;wait 200 microseconds to enter 
                                        ;...program cycle
        jsr     program_complete        ;wait for programming cycle to 




;-------------------------------------------------------------------------
;   lastly, move another 64 bytes - high bytes of last word followed
;                                 by 21 words
;-------------------------------------------------------------------------
        jsr     disable_protect

        rep     #8
        lsr     A       
        move            A1,x:(r2)+      ;move high byte of last word


        do      #21,move_codeC          ;move 63 bytes
        move    P:(r0)+,A		;get 3 byte word
        move            A1,x:(r2)+	;move low  byte to FLASH EEPROM
        rep     #8
        lsr     A       
        move            A1,x:(r2)+      ;move mid  byte to FLASH EEPROM
        rep     #8
        lsr     A       
        move            A1,x:(r2)+      ;move high byte
move_codeC

        jsr     delay_200u              ;wait 200 microseconds to enter 
                                        ;...program cycle
        jsr     program_complete        ;wait for programming cycle to 
                                        ;...finish

        jsr     enable_protect
	rts
;***************************************************************************
;************************************************************************
;   WR_FLA_X                                      !!!
;   This routine writes 64 (24-bit) words from DSP X-RAM to FLASH EEPROM
;                                                 !!!
;	enter with:   	r2 pointing to starting EEPROM Byte Address
;      			r0 pointing to first DSP word to save
;
;-------------------------------------------------------------------------
;   first, move 64 bytes - this includes 21 words + low byte of 22nd
;-------------------------------------------------------------------------
WR_FLA_X
        jsr     disable_protect

        do      #21,movexcodeA          ;move 63 bytes
        move    X:(r0)+,A               ;get 3 byte word
        move            A1,x:(r2)+	;move low  byte to FLASH EEPROM
        rep     #8
        lsr     A       
        move            A1,x:(r2)+      ;move mid  byte to FLASH EEPROM
        rep     #8
        lsr     A       
        move            A1,x:(r2)+      ;move high byte
movexcodeA
					;*** 64th byte
        move    X:(r0)+,A               ;get next 3 byte word
        move            A1,x:(r2)+      ;move low  byte

        jsr     delay_200u              ;wait 200 microseconds to enter 
                                        ;...program cycle
        jsr     program_complete        ;wait for programming cycle to 
                                        ;...finish
;-------------------------------------------------------------------------
;   then, move another 64 bytes - mid and high bytes of 22nd word followed
;                                 by 20 words and then the low and mid
;				  bytes of the next word
;-------------------------------------------------------------------------
        jsr     disable_protect

        rep     #8			
        lsr     A       
        move            A1,x:(r2)+      ;move mid  byte to FLASH EEPROM
        rep     #8
        lsr     A       
        move            A1,x:(r2)+      ;move high byte


        do      #20,movexcodeB          ;move 60 bytes
        move    X:(r0)+,A               ;get 3 byte word
        move            A1,x:(r2)+	;move low  byte to FLASH EEPROM
        rep     #8
        lsr     A       
        move            A1,x:(r2)+      ;move mid  byte to FLASH EEPROM
        rep     #8
        lsr     A       
        move            A1,x:(r2)+      ;move high byte
movexcodeB
					;*** 63rd byte ***
        move    X:(r0)+,A               ;get next 3 byte word
        move            A1,x:(r2)+      ;move low byte
        rep     #8
        lsr     A       		;*** 64th byte ***
        move            A1,x:(r2)+      ;move mid  byte to FLASH EEPROM

        jsr     delay_200u              ;wait 200 microseconds to enter 
                                        ;...program cycle
        jsr     program_complete        ;wait for programming cycle to 




;-------------------------------------------------------------------------
;   lastly, move another 64 bytes - high bytes of last word followed
;                                 by 21 words
;-------------------------------------------------------------------------
        jsr     disable_protect

        rep     #8
        lsr     A       
        move            A1,x:(r2)+      ;move high byte of last word


        do      #21,movexcodeC          ;move 63 bytes
        move    X:(r0)+,A               ;get 3 byte word
        move            A1,x:(r2)+	;move low  byte to FLASH EEPROM
        rep     #8
        lsr     A       
        move            A1,x:(r2)+      ;move mid  byte to FLASH EEPROM
        rep     #8
        lsr     A       
        move            A1,x:(r2)+      ;move high byte
movexcodeC

        jsr     delay_200u              ;wait 200 microseconds to enter 
                                        ;...program cycle
        jsr     program_complete        ;wait for programming cycle to 
                                        ;...finish

        jsr     enable_protect
	rts
;***************************************************************************
;************************************************************************
;   WR_FLA_Y            !!! Y-RAM !!!
;   This routine writes 64 (24-bit) words from DSP RAM to FLASH EEPROM
; 
;	enter with:   	r2 pointing to starting EEPROM Byte Address
;      			r0 pointing to first DSP word to save
;
;-------------------------------------------------------------------------
;   first, move 64 bytes - this includes 21 words + low byte of 22nd
;-------------------------------------------------------------------------
WR_FLA_Y
        jsr     disable_protect

        do      #21,moveycodeA          ;move 63 bytes
        move    Y:(r0)+,A               ;get 3 byte word
        move            A1,x:(r2)+	;move low  byte to FLASH EEPROM
        rep     #8
        lsr     A       
        move            A1,x:(r2)+      ;move mid  byte to FLASH EEPROM
        rep     #8
        lsr     A       
        move            A1,x:(r2)+      ;move high byte
moveycodeA
					;*** 64th byte
        move    Y:(r0)+,A               ;get next 3 byte word
        move            A1,x:(r2)+      ;move low  byte

        jsr     delay_200u              ;wait 200 microseconds to enter 
                                        ;...program cycle
        jsr     program_complete        ;wait for programming cycle to 
                                        ;...finish
;-------------------------------------------------------------------------
;   then, move another 64 bytes - mid and high bytes of 22nd word followed
;                                 by 20 words and then the low and mid
;				  bytes of the next word
;-------------------------------------------------------------------------
        jsr     disable_protect

        rep     #8			
        lsr     A       
        move            A1,x:(r2)+      ;move mid  byte to FLASH EEPROM
        rep     #8
        lsr     A       
        move            A1,x:(r2)+      ;move high byte


        do      #20,moveycodeB          ;move 60 bytes
        move    Y:(r0)+,A               ;get 3 byte word
        move            A1,x:(r2)+	;move low  byte to FLASH EEPROM
        rep     #8
        lsr     A       
        move            A1,x:(r2)+      ;move mid  byte to FLASH EEPROM
        rep     #8
        lsr     A       
        move            A1,x:(r2)+      ;move high byte
moveycodeB
					;*** 63rd byte ***
        move    Y:(r0)+,A               ;get next 3 byte word
        move            A1,x:(r2)+      ;move low byte
        rep     #8
        lsr     A       		;*** 64th byte ***
        move            A1,x:(r2)+      ;move mid  byte to FLASH EEPROM

        jsr     delay_200u              ;wait 200 microseconds to enter 
                                        ;...program cycle
        jsr     program_complete        ;wait for programming cycle to 




;-------------------------------------------------------------------------
;   lastly, move another 64 bytes - high bytes of last word followed
;                                 by 21 words
;-------------------------------------------------------------------------
        jsr     disable_protect

        rep     #8
        lsr     A       
        move            A1,x:(r2)+      ;move high byte of last word


        do      #21,moveycodeC          ;move 63 bytes
        move    Y:(r0)+,A               ;get 3 byte word
        move            A1,x:(r2)+	;move low  byte to FLASH EEPROM
        rep     #8
        lsr     A       
        move            A1,x:(r2)+      ;move mid  byte to FLASH EEPROM
        rep     #8
        lsr     A       
        move            A1,x:(r2)+      ;move high byte
moveycodeC

        jsr     delay_200u              ;wait 200 microseconds to enter 
                                        ;...program cycle
        jsr     program_complete        ;wait for programming cycle to 
                                        ;...finish

        jsr     enable_protect
	rts
;***************************************************************************

;----------------------------------------------------------------------------
;  enable_protect - Software data Protection routine
;
;----------------------------------------------------------------------------
enable_protect  
        move    #>$AA,x0
        move            x0,x:$D555   ;Atmel needs $5555
        move    #>$55,x0
        move            x0,x:$AAAA   ;Atmel needs $2AAA
        move    #>$A0,x0
        move            x0,x:$D555   ;writes are now enabled
        rts
           
;---------------------------------------------------------------------------
;  disable_protect - Disables Software Data protection
; 
;  This code writes the appropriate data to the FLASH part that disables
;  the write protection from the FLASH and allows the part to be loaded.
;
;---------------------------------------------------------------------------
disable_protect
        move    #>$AA,x0
        move            x0,x:$D555
        move    #>$55,x0
        move            x0,x:$AAAA
        move    #>$80,x0
        move            x0,x:$D555
        move    #>$AA,x0
        move            x0,x:$D555
        move    #>$55,x0
        move            x0,x:$AAAA
        move    #>$20,x0
        move            x0,x:$D555      ;exit data protect state
        rts

;---------------------------------------------------------------------------
; delay routine to be used to satisfy tBLC time requirement which is
; a minimum of 150 microseconds.
; Write Pulse Width, Write Pulse Width High requires a minimum of
; 3000 Icycles for 40MHz clock.  (50ns/Icycle)
;---------------------------------------------------------------------------
delay_200u  
        rep    #4000      ; delay of 200 micro seconds.
        nop
        rts
   
;---------------------------------------------------------------------------
;  program_complete - routine to poll for the end of the programming cycle
;
;  The AT29C256 will toggle bit 6 on successive reads until the program
;  cycle completes.  When bit 6 stabilizes, the device is no longer in
;  a programming cycle.
;---------------------------------------------------------------------------
program_complete
        btst    #6,x:$8000              ;check the initial status of bit 6
        jcs     check_set
check_clr                               ;---go here if bit 6 is clear---
        jsr     delay_200u              ;no need to hurry, give it a rest
        btst    #6,x:$8000              ;test bit 6...it was clear
        jcs     check_set               ;if it is now set, continue...
        rts
check_set                               ;---go here if bit 6 is set---
        jsr     delay_200u              
        btst    #6,x:$8000              ;test bit 6...it was set
        jcc     check_clr               ;if it is now clear, continue...
        rts

;**************************************************************************
;  this is the bootstrap loader
;  it is loaded into the memory at boot time and then moves the 
;  application code into DSP RAM
;
boot
	org	p:0,p:*
;----------------------------------------------------------------------------
;The following code moves the loader code from the P:RAM.
;----------------------------------------------------------------------------
begin	move	#reloc,r0		; R0 points to loader code.
	move	#begin_address,r1	; R1 points to runtime loader loc.
	do	#load_length,_move_code	; Move loader code to runtime loc.
	move	p:(r0)+,x0		; Read loader code from load    loc.
	move	x0,p:(r1)+		; Write loader code to  runtime loc.
_move_code
;----------------------------------------------------------------------------
;The following code moves the user's code from the flash to P:RAM.
;----------------------------------------------------------------------------
	jmp	load

reloc
	org	p:begin_address,p:
; -------------------------- P: ---------------------------------------------
load	move	#$0000,r0		; copy EEPROM into P:RAM @ $0000
        move    #$D000,r1               ; R1 points to the code in flash
        move    #$0200,r2               ; R2 contains the length of code 
					; in words.
        do      r2,_ldploop1            ; Loop once for each word of code.
        do      #3,_ldploop2            ; Loop 3 times for each word.
	movem	p:(r1)+,a2		; Load byte from flash.
	rep	#8			; Shift eight times to move
	asr	a			; byte into correct position.
_ldploop2
	movem	a1,p:(r0)+		; Stores 24-bit word.
_ldploop1
; -------------------------- X: ---------------------------------------------
loadx   move    #$0000,r0               ; copy EEPROM into X:RAM @ $0000
        move    #$E000,r1               ; R1 points to the data in flash
        move    #$0600,r2               ; R2 contains the length of data 
					; in words.
        do      r2,_ldxloop1            ; Loop once for each data word
        do      #3,_ldxloop2            ; Loop 3 times for each word.
	movem	p:(r1)+,a2		; Load byte from flash.
	rep	#8			; Shift eight times to move
	asr	a			; byte into correct position.
_ldxloop2
        move    a1,x:(r0)+              ; Stores 24-bit word.
_ldxloop1

; -------------------------- Y: ---------------------------------------------
loady   move    #$0000,r0               ; copy EEPROM into Y:RAM @ $0000
        move    #$8000,r1               ; R1 points to the data in flash
        move    #$1420,r2               ; R2 contains the length of data 
					; in words.
        do      r2,_ldyloop1            ; Loop once for each data word.
        do      #3,_ldyloop2            ; Loop 3 times for each word.
	movem	p:(r1)+,a2		; Load byte from flash.
	rep	#8			; Shift eight times to move
	asr	a			; byte into correct position.
_ldyloop2
        move    a1,y:(r0)+              ; Stores 24-bit word.
_ldyloop1
; ---------------------------------------------------------------------------
	jmp	0			; Jump to the reset vector.
load_length	equ	*-load


        end